@@ -26,16 +26,14 @@ methods:
 from __future__ import annotations
 
 import logging
+from collections.abc import Callable, Iterable, Sequence
+from dataclasses import dataclass
 from typing import (
     Any,
-    Callable,
     Generic,
-    Iterable,
     Literal,
     Protocol,
-    Sequence,
     TypeVar,
-    Union,
     overload,
 )
 
@@ -61,65 +59,55 @@ __all__ = ["attribute", "enumeration", "association", "derivedunion", "redefine"
 log = logging.getLogger(__name__)
 
 
+UnlimitedNatural = int | Literal["*"]
+
+
 E = TypeVar("E")
 
 
 class relation_one(Protocol[E]):
-    def get(self, obj) -> E | None:
-        ...
+    def get(self, obj) -> E | None: ...
 
-    def set(self, obj, value: E | None) -> None:
-        ...
+    def set(self, obj, value: E | None) -> None: ...
 
-    def delete(self, obj, value: E | None) -> None:
-        ...
+    def delete(self, obj, value: E | None) -> None: ...
 
     @overload
-    def __get__(self, obj: None, class_=None) -> relation_one[E]:
-        ...
+    def __get__(self, obj: None, class_=None) -> relation_one[E]: ...
 
     @overload
-    def __get__(self, obj, class_=None) -> E:
-        ...
+    def __get__(self, obj, class_=None) -> E: ...
 
-    def __set__(self, obj, value: E | None) -> None:
-        ...
+    def __set__(self, obj, value: E | None) -> None: ...
 
-    def __delete__(self, obj) -> None:
-        ...
+    def __delete__(self, obj) -> None: ...
 
 
 class relation_many(Protocol[E]):
-    def get(self, obj) -> collection[E]:
-        ...
+    def get(self, obj) -> collection[E]: ...
 
-    def set(self, obj, value: E) -> None:
-        ...
+    def set(self, obj, value: E) -> None: ...
 
-    def delete(self, obj, value: E) -> None:
-        ...
+    def delete(self, obj, value: E) -> None: ...
 
     @overload
-    def __get__(self, obj: None, class_=None) -> relation_many[E]:
-        ...
+    def __get__(self, obj: None, class_=None) -> relation_many[E]: ...
 
     @overload
-    def __get__(self, obj, class_=None) -> collection[E]:
-        ...
+    def __get__(self, obj, class_=None) -> collection[E]: ...
 
-    def __set__(self, obj, value: E) -> None:
-        ...
+    def __set__(self, obj, value: E) -> None: ...
 
-    def __delete__(self, obj) -> None:
-        ...
+    def __delete__(self, obj) -> None: ...
 
 
-relation = Union[relation_one, relation_many]
+relation = relation_one | relation_many
+
 
 T = TypeVar("T")
 
-Lower = Union[Literal[0], Literal[1], Literal[2]]
-Upper = Union[Literal[1], Literal[2], Literal["*"]]
+Lower = Literal[0] | Literal[1] | Literal[2]
+Upper = Literal[1] | Literal[2] | Literal["*"]
 
 
 class umlproperty:
@@ -138,7 +126,7 @@ class umlproperty:
     upper: Upper = 1
 
     def __init__(self, name: str):
-        self.dependent_properties: set[derived | redefine] = set()
+        self.dependent_properties: set[subsettable_umlproperty | redefine] = set()
         self.name = name
         self._name = f"_{name}"
 
@@ -183,16 +171,42 @@ class umlproperty:
             d.propagate(event)
 
 
+class subsettable_umlproperty(umlproperty):
+    """Base class for properties that can be subsetted"""
+
+    subsets: set[umlproperty]
+
+    def __init__(self, name: str):
+        super().__init__(name)
+        self.subsets = set()
+
+    def add(self, subset):
+        if self.upper == 1 and subset.upper == "*":
+            log.warning(
+                "Modeling error: Cannot add a multiplicity * subset "
+                + subset.name
+                + " to a multiplicity 1 superset "
+                + self.name
+            )
+        self.subsets.add(subset)
+        assert isinstance(subset, association | derived | redefine), (
+            f"have element {subset}, expected association, derived or redefine"
+        )
+        subset.dependent_properties.add(self)
+
+    def propagate(self, event): ...
+
+
 class attribute(umlproperty, Generic[T]):
     """Attribute.
 
-    Element.attr = attribute('attr', types.StringType, '')
+    Element.attr = attribute('attr', str, '')
     """
 
     def __init__(
         self,
         name: str,
-        type: type[str | int],
+        type: type[str | int | bool],
         default: str | int | None = None,
     ):
         super().__init__(name)
@@ -215,17 +229,29 @@ class attribute(umlproperty, Generic[T]):
     def set(self, obj, value):
         if (
             value is not None
-            and not isinstance(value, self.type)
-            and not isinstance(value, str)
+            and self.type is not UnlimitedNatural
+            and not isinstance(value, str | self.type)
         ):
-            raise TypeError(
-                f'Value should be of type {hasattr(self.type, "__name__")}'
-                and self.type.__name__
-                or self.type
-            )
+            raise TypeError(f"Value should be of type {self.type}, not {type(value)}")
 
-        if self.type is int and isinstance(value, (str, bool)):
-            value = 0 if value == "False" else 1 if value == "True" else int(value)
+        elif value is not None and self.type is UnlimitedNatural:
+            if value != "*":
+                value = int(value)
+                if value < 0:
+                    raise ValueError(
+                        "Value for UnlimitedNatural should be int >= 0, or '*'"
+                    )
+
+        elif self.type is int and isinstance(value, str | bool):
+            value = (
+                0
+                if value == "False"
+                else 1
+                if value == "True"
+                else "*"
+                if value == "*"
+                else int(value)
+            )
 
         if value == self.get(obj):
             return
@@ -299,7 +325,7 @@ class enumeration(umlproperty):
             self.handle(AttributeUpdated(obj, self, old, self.default))
 
 
-class association(umlproperty):
+class association(subsettable_umlproperty):
     """Association, both uni- and bi-directional.
 
     Element.assoc = association('assoc', Element, opposite='other')
@@ -310,6 +336,51 @@ class association(umlproperty):
 
     If the association is a composite relationship, the association will
     unlink all elements attached to if it is unlinked.
+
+    Associations may be subsets of other associations or sets. The subset must have a
+    multiplicity less than or equal to the superset.
+
+    The logic of maintaining the set contets is complex. While this logic is constrained by set
+    theory, there are cases that arise in which set theory alone would allow multiple outcomes.
+    Making the outcomes deterministic requires making some policy choices. In the cases below,
+    those marked with ***POLICY*** are not solely determined by the logic of set theory.
+
+    The handling of the subset-superset relation depends upon the nature of the superset.
+        If the superset is a derived union, the removal of an element from the subset should also remove
+            it from the superset unless the element is present in the superset because of a different subset.
+        If the superset is an ordinary collection, the behavior depends upon the multiplicity of the set.
+            If the superset has an allowed multiplicity > 1, the removal of an element from the subset should not
+                cause the element to be removed from the superset.
+            If the superset has an allowed multiplicity of 1, the removal of an element from the subset should
+                cause the element to be removed from the superset.
+        if the superset is an ordinary collection and an element is removed from the superset, it must
+            also be removed from the subset.
+
+    Subset changes, both sets have a multiplicity of 1:
+        Both sets {}: subset {a1} => superset {a1}
+        Superset {a1}, subset {}: subset {a2} => superset {a2} ***POLICY***
+        Derived Union: Superset {a1}, subset {a1}: subset {} => superset {} ***POLICY***
+        Ordinary Collection: Superset {a1}, subset {a1}: subset {} => superset {a1} ***POLICY***
+        Superset {a1}, subset {a1}: subset {a2} => superset {a2} ***POLICY***
+
+    Superset changes, both sets have a multiplicity of 1:
+        Both sets {}: superset {a1} => subset {}
+        Subset {a1}, superset {a1}: superset {}, => subset {}
+        Superset {a1}, subset {a1}: superset {a2} => subset {}
+
+    Subset changes, superset has multiplicity *, subset has multiplicity 1 or *:
+        Both sets {}: subset {a1} => superset {a1}
+        Superset {a1}, subset {}, subset {a2} => superset {a1, a2}
+        Derived Union: Superset {a1}, subset {a1}: subset {} => superset {} ***POLICY***
+        Ordinary Collection: Superset {a1}, subset {a1}: subset {} => superset {a1} ***POLICY***
+        Derived Union: Superset {a1}, subset {a1}: subset {a2}, => superset {a2} ***POLICY***
+            assuming that a1 is not in another subset of the derived union
+        Ordinary Collection: Superset {a1}, subset {a1}: subset {a2}, => superset {a1, a2} ***POLICY***
+
+    Superset changes, superset has multiplicity *, subset has multiplicity 1 or *:
+        Both sets {}: superset {a1} => subset {}
+        Superset {a1}, subset {a1}: superset {} => subset {}
+        Superset {a1}, subset {a1}, superset {a2} => subset {}
     """
 
     def __init__(
@@ -388,7 +459,7 @@ class association(umlproperty):
     def _set_one(self, obj, value, from_opposite=False) -> None:
         if not (isinstance(value, self.type) or (value is None)):
             raise TypeError(
-                f"Value should be of type {self.type.__name__}, got a {type(value)} instead"
+                f"Value should be of type {self.type.__modeling_language__}:{self.type.__name__}, got a {type(value)} instead"  # type: ignore[attr-defined]
             )
 
         old = self._get_one(obj)
@@ -414,8 +485,12 @@ class association(umlproperty):
     def _set_many(
         self, obj, value, index, from_opposite=False, from_load=False
     ) -> None:
+        if not value:
+            return None
         if not isinstance(value, self.type):
-            raise TypeError(f"Value should be of type {self.type.__name__}")
+            raise TypeError(
+                f"Value should be of type {self.type.__modeling_language__}:{self.type.__name__}"  # type: ignore[attr-defined]
+            )
 
         # Set the actual value
         c: collection = self._get_many(obj)
@@ -447,13 +522,18 @@ class association(umlproperty):
             opposite = getattr(type(value), self.opposite)
             if not opposite.opposite:
                 opposite.stub = self
-            opposite.set(value, obj, from_opposite=True)
+            if not isinstance(opposite, derived):
+                opposite.set(value, obj, from_opposite=True)
+            else:
+                log.warning(
+                    f"Cannot set opposite of association {self.name} since the opposite {opposite.name} is derived"
+                )
         elif not self.opposite:
             if not self.stub:
                 self.stub = associationstub(self)
                 # Do not let property start with underscore, or it will not be found
                 # as an umlproperty.
-                setattr(self.type, "GAPHOR__associationstub__%x" % id(self), self.stub)
+                setattr(self.type, f"GAPHOR__associationstub__{id(self):x}", self.stub)
             self.stub.set(value, obj)
 
     def delete(self, obj, value, from_opposite=False, do_notify=True):
@@ -477,6 +557,9 @@ class association(umlproperty):
         else:
             if do_notify:
                 self.handle(AssociationSet(obj, self, value, None))
+            for subset in self.subsets:
+                if value == subset.get(obj):
+                    subset.delete(obj, value)
 
     def _del_many(self, obj, value, from_opposite=False, do_notify=True):
         if not value:
@@ -495,6 +578,10 @@ class association(umlproperty):
             else:
                 if do_notify:
                     self.handle(AssociationDeleted(obj, self, value, index))
+                for subset in self.subsets:
+                    col: collection[umlproperty] | umlproperty | None = subset.get(obj)
+                    if isinstance(col, collection) and value in col:
+                        subset.delete(obj, value)
 
             # Remove items collection if empty
             if not items:
@@ -518,6 +605,39 @@ class association(umlproperty):
                 if composite:
                     value.unlink()
 
+    def propagate(self, event):
+        if event.property not in self.subsets:
+            return
+        if not isinstance(event, AssociationUpdated):
+            return
+
+        current_value = self.get(event.element)
+
+        upper_gt_1 = self.upper == "*" or int(self.upper) > 1
+        if isinstance(event, AssociationSet):
+            if event.new_value != current_value:
+                # new_value is coming from a subset. If the value is None and self is not derived
+                # and has upper > 1 then we make no change to this association.
+                not_derived = not isinstance(self, derived)
+                null_value = event.new_value is None
+                if not (not_derived and null_value and upper_gt_1):
+                    self.set(event.element, event.new_value)
+        elif isinstance(event, AssociationDeleted):
+            value = self.get(event.element)
+            if isinstance(value, collection):
+                if event.old_value in self.get(event.element):
+                    # new_value is coming from a subset. If self is not derived and this is a collection
+                    # we make no change to this association.
+                    if isinstance(self, derived) or not upper_gt_1:
+                        self.delete(event.element, event.old_value)
+            elif event.old_value is value:
+                # new_value is coming from a subset. If self is not derived and this is a collection
+                # we make no change to this association.
+                if isinstance(self, derived) or not upper_gt_1:
+                    self.delete(event.element, event.old_value)
+        elif isinstance(event, AssociationAdded):
+            self.set(event.element, event.new_value)
+
 
 class AssociationStubError(Exception):
     pass
@@ -533,7 +653,7 @@ class associationstub(umlproperty):
     """
 
     def __init__(self, association: association):
-        super().__init__("stub_%x" % id(self))
+        super().__init__(f"stub_{id(self):x}")
         self.association = association
 
     def __get__(self, obj, class_=None):
@@ -573,16 +693,16 @@ class associationstub(umlproperty):
             c.discard(value)
 
 
+@dataclass
 class unioncache:
     """Small cache helper object for derivedunions."""
 
-    def __init__(self, owner: object, data: object, version: int) -> None:
-        self.owner = owner
-        self.data = data
-        self.version = version
+    owner: object
+    data: object
+    version: int
 
 
-class derived(umlproperty, Generic[T]):
+class derived(subsettable_umlproperty, Generic[T]):
     """Base class for derived properties, both derived unions and custom
     properties.
 
@@ -596,7 +716,7 @@ class derived(umlproperty, Generic[T]):
 
     opposite = None
 
-    def __init__(  # type: ignore[misc]
+    def __init__(  # type: ignore[explicit-any]
         self,
         name: str,
         type: type[T],
@@ -611,18 +731,10 @@ class derived(umlproperty, Generic[T]):
         self.lower = lower
         self.upper = upper
         self.filter = filter
-        self.subsets: set[umlproperty] = set()
 
         for s in subsets:
             self.add(s)
 
-    def add(self, subset):
-        self.subsets.add(subset)
-        assert isinstance(
-            subset, (association, derived)
-        ), f"have element {subset}, expected association"
-        subset.dependent_properties.add(self)
-
     def load(self, obj, value):
         raise ValueError(
             f"Derivedunion: Properties should not be loaded in a derived union {self.name}: {value}"
@@ -790,7 +902,9 @@ class derivedunion(derived[T]):
         elif isinstance(event, AssociationSet):
             old_value, new_value = event.old_value, event.new_value
             if old_value and old_value not in values:
-                self.handle(DerivedDeleted(event.element, self, old_value))
+                pass
+                # Change so that deletions from subsets no longer delete from superset
+                # self.handle(DerivedDeleted(event.element, self, old_value))
             if new_value and new_value not in values:
                 self.handle(DerivedAdded(event.element, self, new_value))
 
@@ -825,24 +939,28 @@ class redefine(umlproperty):
         name: str,
         type: type[T],
         original: relation,
+        opposite: str | None = None,
     ):
         super().__init__(name)
-        assert isinstance(
-            original, (association, derived)
-        ), f"expected association or derived, got {original}"
+        assert isinstance(original, association | derived), (
+            f"expected association or derived, got {original}"
+        )
         self.decl_class = decl_class
         self.type = type
         self.original: association | derived = original
         self.upper = original.upper
         self.lower = original.lower
+        self._opposite = opposite
 
         original.dependent_properties.add(self)
 
     @property
     def opposite(self) -> str | None:
-        return (
-            self.original.opposite if isinstance(self.original, association) else None
-        )
+        if self._opposite:
+            return self._opposite
+        elif isinstance(self.original, association):
+            return self.original.opposite
+        return None
 
     @property
     def composite(self) -> bool:
@@ -875,7 +993,7 @@ class redefine(umlproperty):
     def set(self, obj, value, from_opposite=False):
         if not (isinstance(value, self.type) or (self.upper == 1 and value is None)):
             raise TypeError(
-                f"Value should be of type {self.type.__name__}, got a {type(value)} instead"
+                f"Value should be of type {self.type.__modeling_language__}:{self.type.__name__}, got a {type(value)} instead"  # type: ignore[attr-defined]
             )
         assert isinstance(self.original, association)
         return self.original.set(obj, value, from_opposite=from_opposite)
